function [A E] = Block_PCA(D,W,lambda1,lambda2)
% Block PCA: find block & low rank structure from a data matrix D
% this function employs CVX toolbox

% Inputs:
%  D: the data matrix
%  W: the weighted matrix
%  lambda1,lambda2: positive weights 
% Outputs:
%  A: the block low rank matrix;
%  E: the matrix consisting errors/noise;

%    min |A|_* + lambda1 * \sum_{i,j} w_{i,j}|A_{i,j}| + lambda2*|E|_1
%    s.t. A + E = D


[m n] = size(D);
ones_mn = ones(m,n);

cvx_solver sdpt3

    % solve the convex programming about A and B;
    cvx_begin SDP
        variable W1(m,m) symmetric;
        variable W2(n,n) symmetric;
        variables A(m,n) E(m,n);    
        minimize(0.5*(trace(W1)+trace(W2)) + lambda1* trace(W'*abs(A)) + lambda2* trace(ones_mn'*abs(E)) );
        subject to
            [W1 A; A' W2] >=0;
            A + E == D;
    cvx_end
    
    
    