function [A E] = Block_PCA_SDPNAL(D,W,lambda1,lambda2)
% solve Block PCA by SDPNAL
% min_{A,E} =   |A|_{*} + lambda_1 <W,|A|>  + lambda_2 |E|_1
% s.t.  D = A + E;
%  If we assume (each element of ) A is nonnegative, (each element of ) B is nonnegative, then      
%
% min_{A, A1,E,W1,W2} =  0.5*[trace(W1)+trace(W2) ]  + lambda_1 <W,A1>  +  lambda_2 < I, E>
% s.t.  [W1  A;
%        A'  W2]  in S^{n}_{+}  (positive semi-definite)
%       A - A1     = 0;
%           A1 + E = D;
%       E(i,j) >= 0;   for all i,j
%       A1(i,j) >= 0;  for all i,j

[m n]= size(D);
n1 = m*n;
blk{1,1} = 's';
blk{1,2} = m+n;
blk{2,1} = 'l';
blk{2,2} = n1;
blk{3,1} = 'l';
blk{3,2} = n1;

C{1} = 0.5*speye(m+n);
C{2} = lambda1*reshape(W,n1,1);
C{3} = lambda2*ones(n1,1);

% assign At{1,1}
n2 = (m+n)*(m+n+1)/2;
t1 = sqrt(2)*0.5;
id_v = 1:n1;
At{1,1} = sparse(  m*(m+1)/2+id_v+(floor(id_v/m).*(floor(id_v/m)+1))/2, id_v, ones(n1,1)*t1, n2, 2*n1);

At{2,1} = [-speye(n1) speye(n1)];
At{3,1} = [sparse(n1,n1) speye(n1)];

b = [sparse(n1,1); reshape(D,n1,1)];

OPTIONS.printlevel = 2;
OPTIONS.proximal = 1; 
OPTIONS.maxitersub= 50;

%%%
clear('D','W','id_v');
%%%

% solve the SDP program 
[obj,X] = sdpNAL(blk,At,C,b,OPTIONS);
%[obj,X] = sdpt3(blk,At,C,b);

A = reshape(X{2},m,n);
E = reshape(X{3},m,n);