function [id_cluster k A] = cluster_blockPCA(L,percentage,theta,expect_community_radius,k)
% identify communities by Block PCA
% Inputs: 
%   L: the distance matrix, L{i,j} indicates the distance between node i
%       and node j, L{i,j}>=0, for all i,j;
%   percentage: Optional, a scalar in (0,1), indicate the weight of dominated sigular
%       values;
%   theta: Optional, a scalar in (0,1), indicate the lambda1:= theta*lambda2;
%   community_radius: Optional, expected community radius;
%   k: Optional, the number of clusters;
% Outputs:
%   id_cluster: a column vecter, indicating the index of each points;
%   k: the number of clusters;
%   A: the low rank structure of the graph

if nargin<=1
    percentage = 0.99;
end
if nargin<=2
    theta = 0.5;
end

n = size(L,1); % In fact, L should be a square matrix;
D = ones(n,n);
if nargin<=3
    expect_community_radius = max(max(L))/3;
end

W = (L./expect_community_radius).^2;
W = W - diag(diag(W)); % set the diagnal elements of W as 0;

% arguments:
lambda2 = 1/sqrt(n);
lambda1 = theta*lambda2;

%[A] = Block_PCA_SDPNAL(D,W,lambda1,lambda2);
%[A] = Block_PCA(D,W,lambda1,lambda2);
[A] = Block_PCA_admm(D,W,lambda1,lambda2);
if nargin<=4
    [id_cluster k]= indicate_cluster(A,percentage);
else
    [id_cluster k]= indicate_cluster(A,percentage,k);
end

end