function draw_matrix_imagesc(varargin)
% draw matrices as images
% Inputs
%   varargin: 
%    A, B, C,...: matrices to be shown;
%    magnitude: a positive  indicate the gray pixel interval: [-magnitude magnitude]
%    title_cell: cell array containing the titles of the matrices;
%    win_split: optional, a 1-by-1 structure, containing the two attributes 'nRow'
%       and 'nCol' which value are positive integers. The window will be
%       broken into nRow - by - nCol small windows to show the images.
%       If just a single of the two attributes assigned, the other will be
%       chosen automatically. 
%    subwindow_pos: optional, a cell array consists the position of each
%       subwindow;

matrix_cell = cell(nargin,1); % containing matrices to be shown;
magnitude = [];
title_cell = [];
win_split = [];
subwindow_pos  = [];
m = []; n = [];
%m_n = [];


% fix the arguments according to the class of input arguments
jj=1;
for ii=1:nargin
    if isempty(magnitude) && isnumeric(varargin{ii}) && length(varargin{ii})==1 && varargin{ii}>0 % magnitude is a scalar
        magnitude = varargin{ii};
    elseif isempty(title_cell) && iscellstr(varargin{ii}) % title_cell is a string cell 
        title_cell = varargin{ii};
    elseif isempty(win_split) && isstruct(varargin{ii}) && length(varargin{ii})==1 %win_split is a 1-by-1 structure
        win_split = varargin{ii};
    elseif isnumeric(varargin{ii}) && ~isvector(varargin{ii}) % matrix to be shown is numeric
        matrix_cell(jj) = varargin(ii);
        jj = jj+1;
    elseif iscell(varargin{ii}) && isvector(varargin{ii}{1}) % cell array with each one element an array containing size for subwindows
        subwindow_pos = varargin{ii};
    elseif iscell(varargin{ii}) && ~iscellstr(varargin{ii}) % cell array containing matrices to be shown
        len_cell = length(varargin{ii});
        matrix_cell(jj:jj+len_cell-1) = varargin{ii}(:);
        jj = jj+len_cell;
    end
end

if jj<=nargin
    matrix_cell(jj:nargin) = []; % delete the empty element at the tail 
end

if isempty(matrix_cell)
    fprintf('No matrix to draw as an image.');
    return
end

if isempty(magnitude)
    magnitude = max(max(abs(matrix_cell{1})));
end

if isempty(subwindow_pos)
    len_matrix_cell = length(matrix_cell);
    subwindow_pos = num2cell(1:len_matrix_cell);
end
    
% set the argument m, n: the image window will be broken into m-by-n
% subwindows to show images
if ~isempty(win_split)
    %m = m_n(1);    n = m_n(2);
    win_split_fieldnames = fieldnames(win_split);
    isfield_nCol = strcmpi('nCol',win_split_fieldnames);
    isfield_nRow = strcmpi('nRow',win_split_fieldnames);
    id_nCol = find(isfield_nCol,1);
    id_nRow = find(isfield_nRow,1);
    if ~isempty(id_nCol)    
        n = win_split.(win_split_fieldnames{id_nCol});
    end      
    if ~isempty(id_nRow)
        m = win_split.(win_split_fieldnames{id_nRow});
    end
end

len_title_cell = length(title_cell);
len_matrix_cell = length(matrix_cell);

if ~isempty(m) && isempty(n)
    n = ceil(len_matrix_cell/m);
elseif isempty(m) &&~isempty(n)
    m = ceil(len_matrix_cell/n);
elseif isempty(m) && isempty(n) % both m and n is empty
    ecc = 0.38;
    [m n] = goldenScale(length(matrix_cell),ecc);
end

% draw the matrices
clims = [-magnitude magnitude];
hold on
colormap(gray);
for ii = 1:len_matrix_cell
    %subplot(m,n,ii);
    subplot(m,n,subwindow_pos{ii});    
    imagesc(matrix_cell{ii},clims);
    if ii<=len_title_cell
        title(title_cell{ii});
    end
end
hold off

end % end of the function draw_matrix_imagesc()


function [nRow nCol] = goldenScale(k,ecc)
  % choose integers nRow and nCol such that nRow * nCol near (and no less
  %   than) k
  % ecc = nRow/nCol
  
  nCol = ceil(sqrt(k/ecc));
  nRow = ceil(k/nCol);
end
