function A = geneNetwork(X,W,gamma1,gamma2,coef)
% sovle the gene regulatory network
%  min_{B} 0.5||B*X1 - X2||^2 + gamma1 ||B||_1 + gamma2 <|B|, W>
%  s.t. sum_{j=1,...,n} |B(i,j)| <= coef;   for all i = 1,...,n
% where
%                 B = coef*A,            with scalar coef > 0
%  A and B in R^{n,n}, X = [x_1, ..., x_m] in R^{n,m}, X1,X2 are the matrices 
%  consist of the first and last n-1 columns of X.

% Reference.
%   This model origins from
%   [1] Fang-Xiang Wu, Li-Zhi Liu and Zhang-Hang Xia, Identification of gene
%       regulatory networks from time course gene expression data.

[n m] = size(X);
A = zeros(n,n);

for ii=1:n
%    x = [];
    d_inverse = 1./(ones(n,1) + gamma2/gamma1*W(ii,:)');
    B = bsxfun(@times,X(:,1:m-1)',d_inverse');
    x = solveLS_l1(B,X(ii,2:m)',d_inverse,gamma1,coef);
    if isempty(x) % failed to solve x 
    	x = solveLS_l1_cvx(B,X(ii,2:m)',d_inverse,gamma1,coef); % solve  the problem by cvx
    end
    if isempty(x) % both cvx and mosek failed
        error('failed to solve the constrained least squares for finding the gene network.');
    end
    A(ii,:) = (x.*d_inverse)'/coef;    
end

end
