function [A id_cluster k]= geneNetwork_community(X,gamma1_v,gamma1_terminate,varargin)
% find communities of gene network
% Inputs:
%  X:=[x_1,...,x_m] in R^{n,m} the observed representation levels of n
%   genes at m time points;
%  gamma1_v: an argument vector, find possible gene networks with different
%   arguments gamma1;
%  gamma1_terminate:  argument of gamma for terminate;
%  coef_A: Optional, a coefficient for A;
% Outputs:
%  A: the adjacent matrix of the gene network, A(i,j) indicate the
%   interaction level between gene i and gene j;
%  id_cluster: a column vecter, indicating the index of each points;
%  k: the number of clusters;

debug_on = 0; % 1 or 0, whether on debug mode

if nargin<=3
    coef_A = 1.0;
else
    coef_A = varargin{1};
end

% arguments
ite_max = 3; % maximum iteration number
c_penalty = 10;
stop_tol = 0.1;
percentage = 0.99;
theta = 0.2;
low_rep_level = 0.1;
  %  lowest interaction, the value less than this number will be intepreted
  %    as no interaction;   
%flag_direct = 0;  % 1: directed graph; 0: undirected graph;
diameter = 10;

len_gamma1_v = length(gamma1_v);
[n] = size(X,1);
num_ite_community = 1; 
A1 = zeros(n,n,num_ite_community*len_gamma1_v);
ite = 0;
W = zeros(n,n);
E = ones(n,n);
flag_terminate = 0; % indicate wheter iterations have stopped;


while 1
    % Step 1. calculate genenetwork
    if flag_terminate % at last iteration, only calulate a single possible matrix A
        len_gamma1_v = 1; gamma1_v = gamma1_terminate;
        A1 = zeros(n,n,1);
    end
    
    for ii=1:len_gamma1_v
        gamma1 = gamma1_v(ii);
        gamma2 = c_penalty*gamma1;
        %A1(:,:,ii) = geneNetwork_cvx(X,W,gamma1,gamma2);
        A1(:,:,ii) = geneNetwork(X,W,gamma1,gamma2,coef_A);
    end
    [A IA] = max(abs(A1),[],3);
    I = ones(n,1)*(1:n);
    ind_mat = sub2ind(size(A1),I',I,IA);
    A = reshape(A1(reshape(ind_mat,n*n,1)),n,n);    
      
    % Step 2. update weighted matrix W
      % 2.1 calulate distance matrix 
    A(abs(A)<=low_rep_level) = 0; 
    %[D]= shortPathLength(phi(abs(A)+abs(A)'),flag_direct);    % undirected graph
    [D]= shortPathLength(phi(abs(A)),1,diameter);  % calculte the distance matrix of the directed graph
      % 2.2 exclude the elements of small values of D
    D(abs(D)<low_rep_level) = 0;      
      % 2.3 deal with the zero elements of D, which indicate infinite distance;
    diameter = max(max(D)); % max(max(D)) the diameter of the graph indicated
                               % by matrix D (note that there are special
                               % elements 0) 
    D(D==0) = diameter; 
      % 2.4 deal with the columns of D with all the elements are DIAMETER
    i_col = min(D)==diameter;    
    D(:,i_col) = diameter/4;
           
      % 2.5 do cluster
    [id_cluster k L] = cluster_blockPCA(D, percentage, theta,diameter/4);
    W1 = abs(E-L); % L is the low rank structure of the graph;
      % 2.6 deal with the case that some elements of columns are all near 1, exclude the chance of
      %     find edges there
    i_col = min(W1)>=0.70;    
    W1(:,i_col) = 0;      
    if debug_on
        fprintf('iteration: \t%d\n',ite);
        fprintf('diameter:\t%.2f\n',diameter);
        fprintf('errer of W:\t%.2f\n',norm(W1-W,1));     
         %A  %D %W1
    end 
        
    if flag_terminate
        break;
    end
    ite = ite + 1; % iterates to update W
    if norm(W1-W,1)< stop_tol
        flag_terminate = 1;
    end
    if ite>=ite_max
        flag_terminate = 1;
    end
    W = W1;    
end
end

function A = phi(A)
% phi: [0,1] --> (0,1]
epsilon = 1.0E-6;
A(abs(A)<epsilon) = 0; % note that the elements 0 indicate the distance if +Inf 
                        % i.e. the elements 0 of X are special labels.
A = exp(-A);
A(A==1) = 0;
end
