function [X coef_A]= generateRepMatrix(A,m,noiseDensity)
% generate gene expression data matrix 
% Inputs/Outputs:
%  m: generate X with m columns
%  noiseDensity: Optional, a scalar in [0,1] the density of incorperated
%    normally distributed random noise, default 0, 
%  coef_A: a coefficient (scalar) to balance  the magnitudes of X(:,i) (not too small, or too large)
%  A, coef_A: (A is a square matrix)
%       X(:,1)  = x1;
%       X(:,i+1) = coef_A*A * X(:,i), i=1,...,m-1;

if nargin<=2
    noiseDensity = 0;
end
n = size(A,1); % A is a square matrix
x1 = rand(n,1);

% initialization
X = zeros(n,m);
xk = x1;
coef_A = 1.0;
norm_x1 = norm(x1);
bound_norm_x = 100;

% calculate a proper coefficient coef_A
for ii=1:m-1
    xk = coef_A*(A*xk);
    if mod(ii,10) == 0 || ii==m-1
        % represent the coefficient cof_A
        norm_xk = norm(xk);
        if norm_xk/norm_x1 >= bound_norm_x || ...
           norm_x1/norm_xk >= bound_norm_x
            coef_A = coef_A/(norm_xk).^(1/ii); 
                % this updating  ensure that xk' satisfies that norm(xk')=1;
        end
    end    
end

% generate X
X(:,1) = x1;
for ii=1:m-1
    X(:,ii+1) = coef_A*(A*X(:,ii));
end

if noiseDensity>0
    X = X + sprandn(n,m,noiseDensity);
end

end
