/*=================================================================
 * This is a MEX-file for MATLAB
 * multi.c 
 * Calculate the multiplication of two matrice A, B
 * Usage:
 *	   H = multi(A,B),  where A in R^{m,p};    B in R^{p,n}, the elements of A, B are all nonnegative
 *     
 *     H(i,j) = min+  { A_{i,k} + B(k,j) | 1<=k<=p, A_{i,k}>0, B(k,j)>0 };  for i~= j;
 *     H(i,j) = 0; for i==j        
 *   where 'min+' indicate taking the minimum over the positive values;
 *============================================================*/

#include <stdlib.h>
#include <math.h>
#include <mex.h>

#define min(a , b) ((a) <= (b) ? (a) : (b))


void mexFunction(int nlhs,       mxArray *plhs[],
		 int nrhs, const mxArray *prhs[])
{
    /* Declare variables */ 
    int i,j,k,m,n,dim;
    double temp, *pr1, *pr2,  *pind;
    double eps = 1.0E-7; 
    
    /* Check for proper number of input and output arguments */    
    if (nrhs < 2) {
	    mexErrMsgTxt("Two input argument required at least.");
    }
    if (nlhs > 1){
	    mexErrMsgTxt("Too many output arguments.");
    }
	  /* The input two matrices must have the same rows number. */
	  if(mxGetN(prhs[0])!=mxGetM(prhs[1])){
	    mexErrMsgTxt("The row number of the first input matrix  must be the same with the column number of the second input matrix.");
	  }
    /* The input two matrices must both two-dimensional */
    if(mxGetNumberOfDimensions(prhs[0])!=2 || mxGetNumberOfDimensions(prhs[1])!=2){
        mexErrMsgTxt("The input two matrices must both two-dimensional.");
    }      
    
    
    /* Get the data */
    pr1=(double *)mxGetPr(prhs[0]);
    pr2=(double *)mxGetPr(prhs[1]);
    
    
    /* Get the row number and column number of the two input matrices */
    dim = mxGetN(prhs[0]);
    m = mxGetM(prhs[0]);
    n = mxGetN(prhs[1]);
    
    /* Allocate the space for the return argument */
    plhs[0]=mxCreateDoubleMatrix(m,n,mxREAL);
    pind=mxGetPr(plhs[0]);
	
		for(i=0;i<m;i++)
		{
			for(j=0;j<n;j++)
			{
                if(j==i)
                {
                    pind[i*dim+j] = 0;
                    continue;
                }    
				temp = 0;
				for(k=0;k<dim;k++)
				{
					if(pr1[i*dim+k]>eps &&pr2[k*n+j]>eps) // the elements are ordered by rows
					{
							if(!temp) temp = pr1[i*dim+k]+ pr2[k*n+j];
							else 
								temp = min(temp, pr1[i*dim+k]+ pr2[k*n+j]);	
				  }
				}
				pind[i*dim+j] = temp;  
			}
		}
	
	
} // end of the function
