function [L flag ite]= shortPathLength(A,flag_direct,max_ite_num)
% calculate the shortest path length of each two vertices of a given
%   undirectional graph
% Inputs:
%   A: the adjacent matrix of the graph, A(i,j) is the length/distance
%      between the vertices i and j; A(i,j) = 0 if there is no edge between
%      the vertices i and j;
%   flag_direct: Optional, indicate whether A is a directed graph; default
%    0 (undirected graph);
%   max_ite_num: Optional, maximum iteration number, i.e. the diagmeter of
%       the input graph; this argument is strongly recommended given as the
%       diameter (or a larger scalar) for  the case of directed graph;
% Outputs:
%   L: a matrix with the same size as matrix A, L(i,j) indicates the
%      shortest path length of the vertices i and j;
%      0: 
%   flag: 1, not reach the maximum iteration number;
%         0, reach the maximum iteration number
%   ite: iteration number

% Revise date: July 10, 2011
% Person in charge: Liang Xi-Jun
% Bug: the shortPathLength() function make mistakes when the graph is
%   directed;
% Revision:
%   (1) add an optional input argument FLAG_DIRECT;
%   (2) revise the variable max_ite_num as an optional input argument;
%   (3)  L ~= 0 is revised to abs(L)>precision; for a small scalar
%       precision;

% Note: this function is not recommended for directed graph, since the
%  worst iteration number is the order of the graph(the number of the
%  nodes); moreover, in this case, the argument max_ite_num is strongly
%  recommended given by user, if the user can estimate the diameter of the
%  graph;


if nargin<=1
    flag_direct = 0;
end
if nargin<=2
    max_ite_num = size(A,1);
end

precision = 1.0E-8;
ite = 0;
A  = full(A);
L = A;

while 1     
      L1 = multi(L,A);
      I_L1 = abs(L1)>precision;
      I_L = abs(L)>precision;
      II = ~I_L & I_L1;
      III = I_L & I_L1;
      
      if ~flag_direct && nnz(II)==0 && nnz(L(III) <= L1(III))==nnz(III)       
          flag = 1;  
          break;
      end
      
      if nnz(II)
          L(II) = L1(II);
      end
      L(III) = min(L(III),L1(III));      
            
      ite =ite +1;      
      if ite>=max_ite_num
          flag = 0;   
          break;          
      end
end

end

% % function B1 = multi(B,A)
% % % A,B are symmetric matrices with the same order;
% % % B1 = B*A whith the operations on scalars overload for matrix multiplication: 
% % %    * --> + ;
% % %    + --> min;
% % 
% % n = length(A);
% % B1 = zeros(n);
% % IB = (B~=0);
% % IA = (A~=0);
% % 
% % for ii =1:n
% %     for jj=ii+1:n
% %        iv = IB(ii,:) & IA(jj,:); 
% %        if nnz(iv)>0
% %            B1(ii,jj)= min( B(ii,iv) + A(jj,iv));
% %            B1(jj,ii) = B1(ii,jj);
% %        end
% %     end
% % end
% % 
% % end