function [x fval] = solveLS_l1(A,b,w,lambda,coef)
% solve constrained l1-regularized least squares
%   min_{x\in R^n}    0.5||A*x-b||^2+  lambda ||x||_1
%   s.t.              <|x|, w> <=coef
%  where lambda>0, coef>0, A \in R^{m,n}, |x| is the vector with the i-th element the absolute
%   value of x(i);

% Inputs
%  A: a matrix in R^{m,n}
%  b: a column vector in R^{m}
%  w: a column vector in R^{n}
%  lambda: a positive scalar
%  coef: a positive scalar
% Outputs
%  x: solution; if failed to solve the problem, x is empty;
%  fval: objactive function value at x;

% construct the matrices and vectors of least squares
n = size(A,2);
H = blkdiag(A'*A,0,zeros(n));
f = [-A'*b; lambda;  zeros(n,1)];
b1 = [zeros(2*n,1); coef; 0];
 
A1 =  [ eye(n)      zeros(n,1) -eye(n);
       -eye(n)      zeros(n,1) -eye(n);
       zeros(1,n)    0         w'     ;
       zeros(1,n)   -1         ones(1,n); ];     % A1 \in R^(2*n+2)

%[x fval] = quadprog(H,f,A1,b1);
[x fval] = quadprog(H,f,A1,b1,[],[],[],[],[],optimset('LargeScale','off','Display','off'));

% add the constant term to  fval 
fval = fval + 0.5*(b'*b);

if ~isempty(x)
    x = x(1:n);
end


end