function [x fval] = solveLS_l1_cvx(A,b,w,lambda,coef)
% solve the constrained l1-regularized least squares by cvx
%
%   min_{x\in R^n}    0.5*||A*x-b||^2+  lambda ||x||_1
%   s.t.              <|x|, w> <= coef
%  where A \in R^{m,n}, |x| is the vector with the i-th element the absolute
%   value of x(i);
%
% Inputs
%  A: a matrix in R^{m,n}
%  b: a column vector in R^{m}
%  w: a column vector in R^{n}
%  lambda: a nonnegative scalar
% Outputs
%  x: solution;
%  fval: objactive function value at x;

n = size(A,2);

cvx_solver sdpt3

    cvx_begin 
        variables x(n,1);    
        minimize  0.5* sum_square(A*x-b) + lambda*norm(x,1);
        subject to  
            w'*abs(x) <= coef;
    cvx_end
    
% calculate the optimal function value    
fval = 0.5* norm(A*x-b)^2 + lambda*norm(x,1);
end


