function generate_cluster_data(dataFileName,num,P_center,noisePercent)
% generate random points for cluster
% Inputs:
%  dataFileName: name of the file to save the genereated data matrix in
%       R^{dim,num}, with each column a generated point
%  num: number of the points
%  P_center: matrix consists the cluster centers, with each column a point 
%  noisePercent: the percent of the number of noise points of the whole points number

[dim clusterNum] = size(P_center);
if num<clusterNum    
    fprintf('warning: the give points number (%d) is less than the cluster number (%d);\n',num,clusterNum);
    return;
end

% 1. generate random points near each clusters
kernelName = 'product';
mag = max(max(abs(P_center)));
Dist_center = kernelMatrix_OnceaLine(kernelName,P_center,P_center);
Dist_center = sqrt(Dist_center);

Dist_center = Dist_center + diag(ones(clusterNum,1)*mag);
r = min(Dist_center)*0.618;

Points = zeros(dim,num);

cluster_size = floor(num*(1-noisePercent)/clusterNum); % number of points at each cluster

ind_p = 1;
for ii=1:clusterNum
    ind_p1 = ind_p + cluster_size-1;
    Points(:,ind_p:ind_p1) = random_points(P_center(:,ii),r(ii),cluster_size);
    ind_p = ind_p1+1;
end

% 2. the  points set randomly at the area
remain_num = num - cluster_size*clusterNum;

if remain_num>0
    lb_mag = min(min(Points(:,1:num-remain_num)));
    ub_mag = max(max(Points(:,1:num-remain_num)));
    Points(:,num-remain_num+1:num) = lb_mag + rand(dim,remain_num)*(ub_mag-lb_mag);
end

save(dataFileName,'Points','num','P_center','clusterNum','dim')

end

function A = random_points(center,radius,num)
    dim = length(center);
    %A = kron(ones(1,num),center)+(rand(dim,num)-0.5)*2*radius;
    V = (rand(dim,num)-0.5)*2;
    t = max(abs(V))./sqrt(sum(V.*V)); % x' := x*t, where t = |x|_{inf} / |x|_2
    A = V*diag(t); % transform the points into a ball: {x in R^n | |x|_2<=1}
    A = kron(ones(1,num),center) + A*radius; 
end