function test_BlockPCA_admm()
% test BlockPCA by admm

flag_test_item = 2;
    % 1: test the role of mu; 2: compare the performance of admm and cvx

%  1.  test the role of the argument mu
if flag_test_item==1

    dataStr = '3_cluster_30_point';
    dataFile = sprintf('../data/clusterPoint/cluster_data_%s.mat',dataStr);
    resultFile = sprintf('../iteration/blockPCA/BlockPCA_admm_%s.mat',dataStr);
    load(dataFile,'Points');

    % calculate D, W
    D = ones(len_P,len_P);
    [W d0] = calculate_W(Points);


    % arguments
    lambda2 = 1/sqrt(length(Points));
    lambda1 = lambda2 * 0.6;
    arg =struct('epsilon_1',1.0E-6,'epsilon_2',1.0E-6,'beta',10/norm(D,'fro'));


    scale_v = [ 0.2 0.4 0.6 0.7 0.8 ];
    percentage = 0.95;

    len_scale = length(scale_v);
    A = cell(len_scale,1);
    E = cell(len_scale,1);
    A1 = A;
    E1  = E;
    id_cluster = cell(len_scale,1);
    num_cluster = zeros(len_scale,1);

    for ii = 1:len_scale
        scale = scale_v(ii);
        lambda1 = lambda2*scale;
        [A{ii} E{ii}] = Block_PCA(D,W,lambda1,lambda2);
        [A1{ii} E1{ii}] = Block_PCA_admm(D,W,lambda1,lambda2,arg);    
         % identify clusters
        [id_cluster{ii} num_cluster(ii)]= indicate_cluster(A1{ii},percentage);    
    end
    save(resultFile,'L','W','d0','lambda1','lambda2','A1','E1','A','E','D','id_cluster','num_cluster','scale_v','percentage');

    for ii = 1:len_scale   
        scale = scale_v(ii);
        colormap(gray);
        imagesc(A1{ii},[-1.2 1.2]);    
        title(sprintf('L (lambda_{1}=%.1f lambda_{2})',scale));
        saveas(gcf,sprintf('../figure/BlockPCA_admm/BlockPCA_admm_%s_lambda1_0_%d_lambda2.fig',dataStr,round(scale*10)));
        saveas(gcf,sprintf('../figure/BlockPCA_admm/BlockPCA_admm_%s_lambda1_0_%d_lambda2.eps',dataStr,round(scale*10)),'psc2');
        close(gcf); % clear the existing figure 
    end

    for ii = 1:len_scale   
        scale = scale_v(ii);    
        % draw the image of clusters
        draw_points_cluster(L,id_cluster{ii});    
        saveas(gcf,sprintf('../figure/BlockPCA_admm/cluster_BlockPCA_admm_%s_lambda1_0_%d_lambda2.fig',dataStr,round(scale*10)));
        saveas(gcf,sprintf('../figure/BlockPCA_admm/cluster_BlockPCA_admm_%s_lambda1_0_%d_lambda2.eps',dataStr,round(scale*10)),'psc2');
        close(gcf);
    end
end

if flag_test_item==2
     % compare the performance of admm and cvx for sovling BlockPCA  
     resultFile = '../iteration/blockPCA_cvx_admm.mat';
     % load data 
     dataStr = 'cluster_data_3_cluster';
     num_v = [30 50 100 200 500 1000]; % numbers of points    
     flag_v = [1 1 1];
     flag_cal_W = 1; % calculate W
     for ii=1:length(num_v)
        num = num_v(ii);
        dataFileName = sprintf('../data/clusterPoint/%s_%d_point.mat',dataStr,num);
        load(dataFileName,'Points');        
        if flag_cal_W || ~ismember('W',who('file',dataFileName))
            [W d0] = calculate_W(Points);
            save(dataFileName,'W','d0','-append');
        else 
            load(dataFileName,'W');
        end            
        fprintf('-------- Points:\t%d.\n',num);
         % arguments and D         
         D = ones(size(W));
         lambda2 = 1/sqrt(length(Points));
         lambda1 = lambda2 * 0.3;
         arg =struct('epsilon_1',1.0E-6,'epsilon_2',1.0E-6,'beta',500/norm(D,'fro'),'ite_max',30000,'mu',5);
         % solving     
         if exist(resultFile,'file')
            load(resultFile,'R_cvx','R_admm','R_sdpnal');
         end
         [R1 R2 R3] = Block_PCA_cvx_admm(D,W,lambda1,lambda2,arg,flag_v);
         if flag_v(1)
             R_cvx{ii} = R1;
         end
         if flag_v(2)
             R_admm{ii} = R2;
         end
         if flag_v(3)
             R_sdpnal{ii} = R3;
         end
         save(resultFile,'R_cvx','R_admm','R_sdpnal');
     end     
     
end


end

function [W, d0] = calculate_W(P)
    % Input: L: the 2-by-n matrix, indicating n points on a plane
    len_P = size(P,2); % number of points    
    W = sqrt(kernelMatrix_OnceaLine('product',P,P));   
    d0 = sum(sum(W))/(1.6*len_P*len_P); %  the average the distances
    W = weight(W,d0);
end

function [varargout] = Block_PCA_cvx_admm(D,W,lambda1,lambda2,arg,flag_v)
    % Inputs
    %  flag_v: a 3-by-1 vector; 
    %    flag_v(1) = 1: test cvx, 0: do not test cvx
    %    flag_v(2) = 1: test admm, 0: do not test admm
    %    flag_v(3) = 1: test sdpnal, 0: do not test sdpnal
    if nargin<=4
        arg = [];
    end
    if nargin<=5
        flag_v = [1; 1];
    end
    % initiation
        R_cvx = struct('L',[],'E',[],'funVal',[],'L_nuclearNorm',[],'L_w_norm',[],...
            'E_1_norm',[],'ite_num',[],'ite_time',[]);
        R_admm = struct('L',[],'E',[],'funVal',[],'L_nuclearNorm',[],'L_w_norm',[],...
            'E_1_norm',[],'ite_num',[],'ite_time',[]);
        R_sdpnal = struct('L',[],'E',[],'funVal',[],'L_nuclearNorm',[],'L_w_norm',[],...
            'E_1_norm',[],'ite_num',[],'ite_time',[]);
    
    % solving
    if flag_v(1)
        tic
        [R_cvx.L R_cvx.E] = Block_PCA(D,W,lambda1,lambda2);
        R_cvx.ite_time =toc;
    end    
    if flag_v(2)
        tic
        [R_admm.L, R_admm.E, exitFlag, R_admm.ite_num] = Block_PCA_admm(D,W,lambda1,lambda2,arg);        
        R_admm.ite_time = toc;
        R_admm.arg = arg;
    end
    if flag_v(3)
        tic
        [R_sdpnal.L, R_sdpnal.E] = Block_PCA_SDPNAL(D,W,lambda1,lambda2);
        R_sdpnal.ite_time = toc;
    end       
    % compare the result   
    if flag_v(1)
        R_cvx.L_nuclearNorm = sum(svd(R_cvx.L));       
        R_cvx.E_1_norm = sum(sum(abs(R_cvx.E)));
        R_cvx.L_w_norm = sum(sum(abs((W.*R_cvx.L))));
        R_cvx.funVal = R_cvx.L_nuclearNorm + lambda1*R_cvx.L_w_norm...
            + lambda2*R_cvx.E_1_norm;
    end
    varargout{1} = R_cvx;
    if flag_v(2)
        R_admm.L_nuclearNorm = sum(svd(R_admm.L));
        R_admm.E_1_norm = sum(sum(abs(R_admm.E)));
        R_admm.L_w_norm = sum(sum(abs((W.*R_admm.L))));
        R_admm.funVal = R_admm.L_nuclearNorm + lambda1*R_admm.L_w_norm...
            + lambda2*R_admm.E_1_norm;        
    end 
    varargout{2} = R_admm;
    if flag_v(3)      
        R_sdpnal.L_nuclearNorm = sum(svd(R_sdpnal.L));
        R_sdpnal.E_1_norm = sum(sum(abs(R_sdpnal.E)));
        R_sdpnal.L_w_norm = sum(sum(abs((W.*R_sdpnal.L))));
        R_sdpnal.funVal = R_sdpnal.L_nuclearNorm + lambda1*R_sdpnal.L_w_norm...
            + lambda2*R_sdpnal.E_1_norm;        
    end 
    varargout{3} = R_sdpnal;
end
