function nci_setup()
% set up the NCI package.
% this function is to be called any time NCI algorithm is stalled on a new
% machine to ensure that the paths are set properly.

% add paths to Matlab serching path
disp('Add paths to Matlab serching path...');
currPath = fileparts(mfilename('fullpath')); % current path
subpaths = {'common','ssvd-code'};
for ii=1:length(subpaths)
    path_str = subpaths{ii};
    addpath([currPath filesep path_str]);
end
savepath; % save the current path

% compile the mex files
flag_fullRecompile = 0; 
    % whether compile all the c files  regardless the existing mex files
curr_folder = pwd; % current folder of the user

cd([currPath filesep 'common']);
cfiles = dir('*.c');
cfiles = {cfiles.name};
has_mex = 1;
if ~flag_fullRecompile
    for k=1:length(cfiles)
       fileName = cfiles{k};
       if exist(fileName(1:end-2),'file') ~= 3
           has_mex = 0;
           break;
       end
    end
end

str_split = '------------------------------------------------';
flag_success = 1;
if flag_fullRecompile || ~has_mex
    disp('Attempting to produce the NCI mex files...');
    disp(str_split);    
    for k=1:length(cfiles)
        fileName = cfiles{k};
        try
            cmd_str = sprintf('%s ','mex',fileName);
            fprintf(1,'%s\n',cmd_str);
            mex(fileName);
        catch
            flag_success = 0;
        end
    end
    if flag_success
        disp(str_split);
    else
        disp('Error: One or more NCI''s required Mex files could not be produced.');
        disp('This is usually because your Mex system has not been properly set up.');
        disp('Please consult the Matlab documentation for details on how to do this.');
    end
end

% set up CVX
disp('Set up CVX...');
cvx_setup_path = [currPath filesep 'cvx' filesep 'cvx' ];
cvx_setup_file = [cvx_setup_path filesep 'cvx_setup.m'];
if ~exist(cvx_setup_file,'file')
    disp('Do not find the cvx set-up file %s',cvx_setup_file);
    flag_success = 0;
else
    cd(cvx_setup_path);
    eval('cvx_setup');
end

cd(curr_folder);
if flag_success
    fprintf('Succeed to set up NCI.\n');
end
end