function [U s V k iter_v ] = ssvd_k_layer(X,param,percentage,k)
% do sparse SVD, generate k layers

% Inputs:
%   X: the object matrix;
%   param: a stucture, see ssvd() for details, default [];
%   percentage: Optional, a scalar that indicate the weight to choose the 
%       dominate sigular values;
%   k: Optional,the number of layers to extract;
% Outputs:
%   U: matrix with each column a left singular vectors (k columns)
%   V: matrix with each column a right singular vectors (k columns)
%   s: a column vector containing the sigular values;
%   k: the number of layers to extract
%   iter_v: a k*1 array, indicating the iteration numbers of extracting each layer

if nargin<=1 
    param = [];
end
if nargin<=2
    percentage = 0.99;
end

[n m] = size(X);
U = zeros(n,1);
V = zeros(m,1);
s = svd(X,'econ');
iter_v = [];
if isempty(s) || norm(s,1)<=1.0E-10
    k=1; s = 0;
    return
end

if nargin<=3
    k = dominateSingularValueNumber(s,percentage);
end

if isempty(k) || ~isnumeric(k) || k<=0
    k = 1;
end
U = sparse(n,k);
V = sparse(m,k);
iter_v = zeros(k,1);

for ii=1:k
    [U(:,ii) V(:,ii) iter_v(ii)] = ssvd(X,param);
    X = X - s(ii)*U(:,ii)*V(:,ii)'; 
end
end % end of the function

function k = dominateSingularValueNumber(s,percentage)
% determine the number of main sigular values
% Inputs:
%  s: a vector containing desendingly ordered sigular values 
%   percentage:  a scalar that indicate the weight to choose the 
%       dominate sigular values;
% Outputs:
%  k: a positive integer indicating that the first k sigular values are
%   dominated ones;

if size(s,1) == 1;
    s = s'; % ensure that s is a column vector;
end
lp = 0.001;
s = s/s(1);
% calulate p(k): the mean value of the first k elements of s
s1 = cumsum(s); % now s1 is a inscreasing series
k1 = find(s>=lp,1,'last');
s1 = s1(1:k1-1)./s1(2:k1); 
k=find(s1>percentage,1,'first');
if isempty(k)||k==0
    k=k1;
end

end
